# Copyright 2022 Alvaro Bartolome, alvarobartt @ GitHub
# See LICENSE for details.

"""`wandbfsspec `: fsspec interface for Weights & Biases (wandb)"""

__author__ = "Alvaro Bartolome, alvarobartt @ GitHub"
__version__ = "0.1.0"
